@@ -54,6 +54,7 @@ class WrapNagios(ServicesCheck):
             dimensions.update({'target_host': socket.getfqdn()})
 
         extra_path = self.init_config.get('check_path')
+        env = {"PATH": extra_path} if extra_path else None
 
         last_run_path = self.init_config.get('temp_file_path')
         # Use a default last_run_file if no temp_file is specified in the YAML
@@ -80,24 +81,33 @@ class WrapNagios(ServicesCheck):
         if self._do_skip_check(instance, last_run_data) is True:
             return
 
+        metric_name = instance.get('metric_name', instance['name'])
+
+        detail = None
         try:
             proc = subprocess.Popen(instance['check_command'].split(" "),
-                                    env={"PATH": extra_path},
+                                    env=env,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
             output = proc.communicate()
             # The check detail is all the text before the pipe
             detail = output[0].split('|')[0]
-            # TODO(dschroeder): Save/send 'detail' when supported by the API
         except OSError:
             # Return an UNKNOWN code (3) if I have landed here
-            self.gauge(instance['name'], 3, dimensions=dimensions)
-            self.log.info(instance['check_command'].split(" ")[0] + " is missing or unreadable")
+            error_string = instance['check_command'].split(" ")[0] + " is missing or unreadable"
+            self.gauge(metric_name,
+                       3,
+                       dimensions=dimensions,
+                       value_meta={'error': error_string})
+            self.log.info(error_string)
             return
 
         status_code = proc.poll()
         last_run_data[instance['name']] = time.time()
-        self.gauge(instance['name'], status_code, dimensions=dimensions)
+        self.gauge(metric_name,
+                   status_code,
+                   dimensions=dimensions,
+                   value_meta={'detail': detail})
         # Return DOWN on critical, UP otherwise
         if status_code == "2":
             return Status.DOWN, "DOWN: {0}".format(detail)
