@@ -11,12 +11,16 @@ log = logging.getLogger(__name__)
 
 
 class Measurement(object):
-    def __init__(self, name, timestamp, value, dimensions, delegated_tenant=None):
+    def __init__(self, name, timestamp, value, dimensions, delegated_tenant=None, value_meta=None):
         self.name = name
         self.timestamp = timestamp
         self.value = value
         self.dimensions = dimensions.copy()
         self.delegated_tenant = delegated_tenant
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
 
 
 class MetricTypes(object):
@@ -46,7 +50,8 @@ class Gauge(Metric):
     """ A metric that tracks a value at particular points in time. """
 
     def __init__(self, formatter, name, dimensions,
-                 hostname, device_name, delegated_tenant=None):
+                 hostname, device_name, delegated_tenant=None,
+                 value_meta=None):
         self.formatter = formatter
         self.name = name
         self.value = None
@@ -55,6 +60,10 @@ class Gauge(Metric):
         self.hostname = hostname
         self.device_name = device_name
         self.timestamp = time()
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
 
     def sample(self, value, sample_rate, timestamp=None):
         self.value = value
@@ -62,20 +71,19 @@ class Gauge(Metric):
 
     def flush(self, timestamp):
         if self.value is not None:
-            res = [self.formatter(
-                metric=self.name,
-                timestamp=self.timestamp or timestamp,
-                value=self.value,
-                dimensions=self.dimensions,
-                delegated_tenant=self.delegated_tenant,
-                hostname=self.hostname,
-                device_name=self.device_name,
-                metric_type=MetricTypes.GAUGE
-            )]
+            value = self.value
             self.value = None
-            return res
-
-        return []
+            return [self.formatter(metric=self.name,
+                                  timestamp=self.timestamp or timestamp,
+                                  value=value,
+                                  dimensions=self.dimensions,
+                                  delegated_tenant=self.delegated_tenant,
+                                  hostname=self.hostname,
+                                  device_name=self.device_name,
+                                  metric_type=MetricTypes.GAUGE,
+                                  value_meta=self.value_meta)]
+        else:
+            return []
 
 
 class Counter(Metric):
@@ -83,7 +91,8 @@ class Counter(Metric):
     """ A metric that tracks a counter value. """
 
     def __init__(self, formatter, name, dimensions,
-                 hostname, device_name, delegated_tenant=None):
+                 hostname, device_name, delegated_tenant=None,
+                 value_meta=None):
         self.formatter = formatter
         self.name = name
         self.value = 0
@@ -91,24 +100,29 @@ class Counter(Metric):
         self.delegated_tenant = delegated_tenant
         self.hostname = hostname
         self.device_name = device_name
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
 
     def sample(self, value, sample_rate, timestamp=None):
         self.value += value * int(1 / sample_rate)
 
     def flush(self, timestamp):
-        try:
-            return [self.formatter(
-                metric=self.name,
-                value=self.value,
-                timestamp=timestamp,
-                dimensions=self.dimensions,
-                delegated_tenant=self.delegated_tenant,
-                hostname=self.hostname,
-                device_name=self.device_name,
-                metric_type=MetricTypes.RATE
-            )]
-        finally:
-            self.value = 0
+        if self.value is not None:
+            value = self.value
+            self.value = None
+            return [self.formatter(metric=self.name,
+                                   value=value,
+                                   timestamp=timestamp,
+                                   dimensions=self.dimensions,
+                                   delegated_tenant=self.delegated_tenant,
+                                   hostname=self.hostname,
+                                   device_name=self.device_name,
+                                   metric_type=MetricTypes.RATE,
+                                   value_meta=self.value_meta)]
+        else:
+            return []
 
 
 class Histogram(Metric):
@@ -116,7 +130,8 @@ class Histogram(Metric):
     """ A metric to track the distribution of a set of values. """
 
     def __init__(self, formatter, name, dimensions,
-                 hostname, device_name, delegated_tenant=None):
+                 hostname, device_name, delegated_tenant=None,
+                 value_meta=None):
         self.formatter = formatter
         self.name = name
         self.count = 0
@@ -126,14 +141,19 @@ class Histogram(Metric):
         self.delegated_tenant = delegated_tenant
         self.hostname = hostname
         self.device_name = device_name
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
 
     def sample(self, value, sample_rate, timestamp=None):
         self.count += int(1 / sample_rate)
         self.samples.append(value)
 
-    def flush(self, ts):
+    def flush(self, timestamp):
+        metrics = []
         if not self.count:
-            return []
+            return metrics
 
         self.samples.sort()
         length = len(self.samples)
@@ -149,31 +169,28 @@ class Histogram(Metric):
             ('count', self.count, MetricTypes.RATE)
         ]
 
-        metrics = [self.formatter(
-            hostname=self.hostname,
-            device_name=self.device_name,
-            dimensions=self.dimensions,
-            delegated_tenant=self.delegated_tenant,
-            metric='%s.%s' % (self.name, suffix),
-            value=value,
-            timestamp=ts,
-            metric_type=metric_type
-        ) for suffix, value, metric_type in metric_aggrs
-        ]
+        metrics.append([self.formatter(hostname=self.hostname,
+                                       device_name=self.device_name,
+                                       dimensions=self.dimensions,
+                                       delegated_tenant=self.delegated_tenant,
+                                       metric='%s.%s' % (self.name, suffix),
+                                       value=value,
+                                       timestamp=timestamp,
+                                       metric_type=metric_type,
+                                       value_meta=self.value_meta
+                                       ) for suffix, value, metric_type in metric_aggrs])
 
         for p in self.percentiles:
             val = self.samples[int(round(p * length - 1))]
             name = '%s.%spercentile' % (self.name, int(p * 100))
-            metrics.append(self.formatter(
-                hostname=self.hostname,
-                dimensions=self.dimensions,
-                delegated_tenant=self.delegated_tenant,
-                metric=name,
-                value=val,
-                timestamp=ts,
-                metric_type=MetricTypes.GAUGE
-            ))
-
+            metrics.append(self.formatter(hostname=self.hostname,
+                                          dimensions=self.dimensions,
+                                          delegated_tenant=self.delegated_tenant,
+                                          metric=name,
+                                          value=val,
+                                          timestamp=timestamp,
+                                          metric_type=MetricTypes.GAUGE,
+                                          value_meta=self.value_meta))
         # Reset our state.
         self.samples = []
         self.count = 0
@@ -186,7 +203,8 @@ class Set(Metric):
     """ A metric to track the number of unique elements in a set. """
 
     def __init__(self, formatter, name, dimensions,
-                 hostname, device_name, delegated_tenant=None):
+                 hostname, device_name, delegated_tenant=None,
+                 value_meta=None):
         self.formatter = formatter
         self.name = name
         self.dimensions = dimensions.copy()
@@ -194,34 +212,41 @@ class Set(Metric):
         self.hostname = hostname
         self.device_name = device_name
         self.values = set()
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
+
 
     def sample(self, value, sample_rate, timestamp=None):
         self.values.add(value)
 
     def flush(self, timestamp):
+        metrics = []
         if not self.values:
             return []
-        try:
-            return [self.formatter(
-                hostname=self.hostname,
-                device_name=self.device_name,
-                dimensions=self.dimensions,
-                delegated_tenant=self.delegated_tenant,
-                metric=self.name,
-                value=len(self.values),
-                timestamp=timestamp,
-                metric_type=MetricTypes.GAUGE
-            )]
-        finally:
+        else:
+            values = self.values.copy()
             self.values = set()
 
+        return [self.formatter(hostname=self.hostname,
+                               device_name=self.device_name,
+                               dimensions=self.dimensions,
+                               delegated_tenant=self.delegated_tenant,
+                               metric=self.name,
+                               value=len(values),
+                               timestamp=timestamp,
+                               metric_type=MetricTypes.GAUGE,
+                               value_meta=self.value_meta)]
+
 
 class Rate(Metric):
 
     """ Track the rate of metrics over each flush interval """
 
     def __init__(self, formatter, name, dimensions,
-                 hostname, device_name, delegated_tenant=None):
+                 hostname, device_name, delegated_tenant=None,
+                 value_meta=None):
         self.formatter = formatter
         self.name = name
         self.dimensions = dimensions.copy()
@@ -229,6 +254,10 @@ class Rate(Metric):
         self.hostname = hostname
         self.device_name = device_name
         self.samples = []
+        if value_meta:
+            self.value_meta = value_meta.copy()
+        else:
+            self.value_meta = None
 
     def sample(self, value, sample_rate, timestamp=None):
         if not timestamp:
@@ -251,18 +280,16 @@ class Rate(Metric):
     def flush(self, timestamp):
         if len(self.samples) < 2:
             return []
-        try:
-            val = self._rate(self.samples[-2], self.samples[-1])
-            self.samples = self.samples[-1:]
-        except Exception:
-            log.warn('Error flushing {0} sample.'.format(self.name))
-            return []
+
+        val = self._rate(self.samples[-2], self.samples[-1])
+        self.samples = self.samples[-1:]
 
         return [self.formatter(hostname=self.hostname,
-           device_name=self.device_name,
-           dimensions=self.dimensions,
-           delegated_tenant=self.delegated_tenant,
-           metric=self.name,
-           value=val,
-           timestamp=timestamp,
-           metric_type=MetricTypes.GAUGE)]
+                               device_name=self.device_name,
+                               dimensions=self.dimensions,
+                               delegated_tenant=self.delegated_tenant,
+                               metric=self.name,
+                               value=val,
+                               timestamp=timestamp,
+                               metric_type=MetricTypes.GAUGE,
+                               value_meta=self.value_meta)]
